"""Example Utility Functions."""


def multiply(first_digit: float, second_digit: float) -> float:
    """Multiplies two floats and returns the result."""
    result = first_digit * second_digit
    return result


def divide(first_digit: float, second_digit: float) -> float:
    """Divides first float by second one and returns the result."""
    result = first_digit / second_digit
    return result
